@@ -110,13 +110,13 @@ setClass("lavModel",          # MATRIX representation of the sem model
         th.idx="list",
 
         nx.free="integer",
-        #nx.unco="integer",
+        nx.unco="integer",
         nx.user="integer",
 
         m.free.idx="list",
         x.free.idx="list",
-        #m.unco.idx="list",
-        #x.unco.idx="list",
+        #m.unco.idx="list", # always the same as m.free.idx
+        x.unco.idx="list",
         m.user.idx="list",
         x.user.idx="list",
         x.def.idx="integer",
@@ -124,6 +124,8 @@ setClass("lavModel",          # MATRIX representation of the sem model
         x.cin.idx="integer",
         x.free.var.idx="integer",
 
+        ceq.simple.only="logical",
+        ceq.simple.K="matrix",
         eq.constraints="logical",
         eq.constraints.K="matrix",
         eq.constraints.k0="numeric",
