@@ -8,10 +8,11 @@
 #
 # Note: this replaces the (old) lav_model_wls_v() function
 #
-# - YR 22 Okt 2017
+# - YR 22 Okt 2017: initial version
 # - YR 03 Dec 2017: add lavh1, implied is either lavimplied or lavh1
 #                   add support for clustered data: first.order
 # - YR 03 Jan 2018: add support for clustered data: expected
+# - YR 23 Aug 2018: lav_model_h1_acov (0.6-3)
 
 lav_model_h1_information <- function(lavobject      = NULL,
                                      lavmodel       = NULL,
@@ -38,9 +39,10 @@ lav_model_h1_information <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # information
+    information <- lavoptions$information[1] # ALWAYS take the first one
+                                             # the caller must control it
 
-    estimator   <- lavmodel@estimator
-    information <- lavoptions$information
 
     # compute information matrix
     if(information == "observed") {
@@ -94,18 +96,42 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
 
     # structured of unstructured? (since 0.5-23)
     if(!is.null(lavoptions) &&
-       !is.null(lavoptions$h1.information) &&
-       lavoptions$h1.information == "unstructured") {
+       !is.null(lavoptions$h1.information[1]) &&
+       lavoptions$h1.information[1] == "unstructured") {
         structured <- FALSE
     } else {
         structured <- TRUE
     }
 
+
     # 1. WLS.V (=A1) for GLS/WLS
     if(lavmodel@estimator == "GLS"  || lavmodel@estimator == "WLS") {
         A1 <- lavsamplestats@WLS.V
     }
 
+    # 1b.
+    else if(lavmodel@estimator == "DLS") {
+        if(lavmodel@estimator.args$dls.GammaNT == "sample") {
+            A1 <- lavsamplestats@WLS.V
+        } else {
+            A1 <- vector("list", length = lavsamplestats@ngroups)
+            for(g in seq_len(lavsamplestats@ngroups)) {
+                dls.a <- lavmodel@estimator.args$dls.a
+                GammaNT <- lav_samplestats_Gamma_NT(
+                               COV            = lavimplied$cov[[g]],
+                               MEAN           = lavimplied$mean[[g]],
+                               rescale        = FALSE,
+                               x.idx          = lavsamplestats@x.idx[[g]],
+                               fixed.x        = lavmodel@fixed.x,
+                               conditional.x  = lavmodel@conditional.x,
+                               meanstructure  = lavmodel@meanstructure,
+                               slopestructure = lavmodel@conditional.x)
+                W.DLS <- (1 - dls.a)*lavsamplestats@NACOV[[g]] + dls.a*GammaNT
+                A1[[g]] <- lav_matrix_symmetric_inverse(W.DLS)
+            }
+        }
+    }
+
     # 2. DWLS/ULS diagonal @WLS.VD slot
     else if(lavmodel@estimator == "DWLS"  || lavmodel@estimator == "ULS") {
         # diagonal only!!
@@ -113,7 +139,7 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
     }
 
     # 3a. ML single level
-    else if( (lavmodel@estimator == "ML" || lavmodel@estimator == "NTRLS")
+    else if( lavmodel@estimator %in% c("ML", "NTRLS", "DLS")
              && lavdata@nlevels == 1L ) {
         A1 <- vector("list", length=lavsamplestats@ngroups)
 
@@ -221,7 +247,7 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
 
             # stochastic group weight
             if(lavmodel@group.w.free) {
-                # unweight!!
+                # unweight!! (as otherwise, we would 'weight' again)
                 a <- exp(lavimplied$group.w[[g]]) / lavsamplestats@nobs[[g]]
                 A1[[g]] <- lav_matrix_bdiag( matrix(a, 1L, 1L), A1[[g]])
             }
@@ -264,6 +290,7 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
         } # g
     } # ML + multilevel
 
+
     A1
 }
 
@@ -296,18 +323,42 @@ lav_model_h1_information_observed <- function(lavobject      = NULL,
 
     # structured?
     if(!is.null(lavoptions) &&
-       !is.null(lavoptions$h1.information) &&
-       lavoptions$h1.information == "unstructured") {
+       !is.null(lavoptions$h1.information[1]) &&
+       lavoptions$h1.information[1] == "unstructured") {
         structured <- FALSE
     } else {
         structured <- TRUE
     }
 
     # 1. WLS.V (=A1) for GLS/WLS
-    if(lavmodel@estimator == "GLS"  || lavmodel@estimator == "WLS") {
+    if(lavmodel@estimator == "GLS"  || lavmodel@estimator == "WLS" ||
+       lavmodel@estimator == "DLS") {
         A1 <- lavsamplestats@WLS.V
     }
 
+    # 1b.
+    else if(lavmodel@estimator == "DLS") {
+        if(lavmodel@estimator.args$dls.GammaNT == "sample") {
+            A1 <- lavsamplestats@WLS.V
+        } else {
+            A1 <- vector("list", length = lavsamplestats@ngroups)
+            for(g in seq_len(lavsamplestats@ngroups)) {
+                dls.a <- lavmodel@estimator.args$dls.a
+                GammaNT <- lav_samplestats_Gamma_NT(
+                               COV            = lavimplied$cov[[g]],
+                               MEAN           = lavimplied$mean[[g]],
+                               rescale        = FALSE,
+                               x.idx          = lavsamplestats@x.idx[[g]],
+                               fixed.x        = lavmodel@fixed.x,
+                               conditional.x  = lavmodel@conditional.x,
+                               meanstructure  = lavmodel@meanstructure,
+                               slopestructure = lavmodel@conditional.x)
+                W.DLS <- (1 - dls.a)*lavsamplestats@NACOV[[g]] + dls.a*GammaNT
+                A1[[g]] <- lav_matrix_symmetric_inverse(W.DLS)
+            }
+        }
+    }
+
     # 2. DWLS/ULS diagonal @WLS.VD slot
     else if(lavmodel@estimator == "DWLS"  || lavmodel@estimator == "ULS") {
         # diagonal only!!
@@ -503,8 +554,8 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
 
     # structured?
     if(!is.null(lavoptions) &&
-       !is.null(lavoptions$h1.information) &&
-       lavoptions$h1.information == "unstructured") {
+       !is.null(lavoptions$h1.information[1]) &&
+       lavoptions$h1.information[1] == "unstructured") {
         structured <- FALSE
     } else {
         structured <- TRUE
@@ -512,8 +563,8 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
 
     # clustered?
     if(!is.null(lavoptions) &&
-       !is.null(lavoptions$clustered) &&
-       lavoptions$clustered) {
+       !is.null(lavoptions$.clustered) &&
+       lavoptions$.clustered) {
         clustered <- TRUE
         if(is.null(lavdata@Lp[[1]])) {
             stop("lavaan ERROR: lavdata@Lp is empty, while clustered = TRUE")
@@ -524,9 +575,9 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
         #if(lavsamplestats@missing.flag) {
         #    stop("lavaan ERROR: clustered information is not (yet) available when missing = \"ML\"")
         #}
-        if(lavmodel@conditional.x) {
-            stop("lavaan ERROR: clustered information is not (yet) available when conditional.x = TRUE")
-        }
+        #if(lavmodel@conditional.x) {
+        #    stop("lavaan ERROR: clustered information is not (yet) available when conditional.x = TRUE")
+        #}
         #if(!structured) {
         #    stop("lavaan ERROR: clustered information is not (yet) available when h1.information = \"unstructured\"")
         #}
@@ -590,22 +641,43 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
 
         } else if(estimator == "ML" && lavdata@nlevels > 1L) {
 
-            MU.W    <- implied$mean[[ (g-1)*lavdata@nlevels + 1L ]]
-            MU.B    <- implied$mean[[ (g-1)*lavdata@nlevels + 2L ]]
-            SIGMA.W <- implied$cov[[  (g-1)*lavdata@nlevels + 1L ]]
-            SIGMA.B <- implied$cov[[  (g-1)*lavdata@nlevels + 2L ]]
+            # if not-structured, we use lavh1, and that is always
+            # 'unconditional' (for now)
+            if(lavmodel@conditional.x && structured) {
+                Res.Sigma.W <- implied$res.cov[[    (g-1)*lavdata@nlevels + 1L]]
+                Res.Int.W   <- implied$res.int[[    (g-1)*lavdata@nlevels + 1L]]
+                Res.Pi.W    <- implied$res.slopes[[ (g-1)*lavdata@nlevels + 1L]]
 
-            # clustered data
-            B1[[g]] <- lav_mvnorm_cluster_information_firstorder(
-                           Y1           = lavdata@X[[g]],
-                           YLp          = lavsamplestats@YLp[[g]],
-                           Lp           = lavdata@Lp[[g]],
-                           Mu.W         = MU.W,
-                           Sigma.W      = SIGMA.W,
-                           Mu.B         = MU.B,
-                           Sigma.B      = SIGMA.B,
-                           x.idx        = lavsamplestats@x.idx[[g]],
-                           divide.by.two = TRUE)
+                Res.Sigma.B <- implied$res.cov[[    (g-1)*lavdata@nlevels + 2L]]
+                Res.Int.B   <- implied$res.int[[    (g-1)*lavdata@nlevels + 2L]]
+                Res.Pi.B    <- implied$res.slopes[[ (g-1)*lavdata@nlevels + 2L]]
+                B1[[g]] <- lav_mvreg_cluster_information_firstorder(
+                               Y1            = lavdata@X[[g]],
+                               YLp           = lavsamplestats@YLp[[g]],
+                               Lp            = lavdata@Lp[[g]],
+                               Res.Sigma.W   = Res.Sigma.W,
+                               Res.Int.W     = Res.Int.W,
+                               Res.Pi.W      = Res.Pi.W,
+                               Res.Sigma.B   = Res.Sigma.B,
+                               Res.Int.B     = Res.Int.B,
+                               Res.Pi.B      = Res.Pi.B,
+                               divide.by.two = TRUE)
+            } else {
+                MU.W    <- implied$mean[[ (g-1)*lavdata@nlevels + 1L ]]
+                MU.B    <- implied$mean[[ (g-1)*lavdata@nlevels + 2L ]]
+                SIGMA.W <- implied$cov[[  (g-1)*lavdata@nlevels + 1L ]]
+                SIGMA.B <- implied$cov[[  (g-1)*lavdata@nlevels + 2L ]]
+                B1[[g]] <- lav_mvnorm_cluster_information_firstorder(
+                               Y1            = lavdata@X[[g]],
+                               YLp           = lavsamplestats@YLp[[g]],
+                               Lp            = lavdata@Lp[[g]],
+                               Mu.W          = MU.W,
+                               Sigma.W       = SIGMA.W,
+                               Mu.B          = MU.B,
+                               Sigma.B       = SIGMA.B,
+                               x.idx         = lavsamplestats@x.idx[[g]],
+                               divide.by.two = TRUE)
+            }
 
         } else if(estimator == "ML" && lavdata@nlevels == 1L) {
 
@@ -701,3 +773,107 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
     B1
 }
 
+
+# asymptotic variance matrix (=Gamma/N) of the unrestricted (H1)
+# sample statistics
+#
+# FIXME: make this work for categorical/GLS/WLS/...
+#
+lav_model_h1_acov <- function(lavobject      = NULL,
+                              lavmodel       = NULL,
+                              lavsamplestats = NULL,
+                              lavdata        = NULL,
+                              lavoptions     = NULL,
+                              lavimplied     = NULL,
+                              lavh1          = NULL,
+                              lavcache       = NULL,
+                              meanstructure  = NULL,  # if specified, use it
+                              h1.information = NULL,  # if specified, use it
+                              se = NULL) {            # if specified, use it
+
+    if(!is.null(lavobject) && inherits(lavobject, "lavaan")) {
+        lavmodel       <- lavobject@Model
+        lavsamplestats <- lavobject@SampleStats
+        lavdata        <- lavobject@Data
+        lavimplied     <- lavobject@implied
+        if(.hasSlot(lavobject, "h1")) {
+            lavh1      <- lavobject@h1
+        } else {
+            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+                                      lavsamplestats = lavobject@SampleStats,
+                                      lavoptions = lavobject@Options)
+        }
+        lavcache       <- lavobject@Cache
+        lavoptions     <- lavobject@Options
+    }
+
+    # override
+    if(!is.null(meanstructure)) {
+        lavoptions$meanstructure <- meanstructure
+    }
+    if(!is.null(h1.information)) {
+        lavoptions$h1.information[1] <- h1.information
+    }
+    if(!is.null(se)) {
+        lavoptions$se <- se
+    }
+
+
+
+    # information
+    information <- lavoptions$information[1] # ALWAYS used the first
+
+    # compute information matrix
+    if(information == "observed") {
+        I1 <- lav_model_h1_information_observed(lavmodel = lavmodel,
+            lavsamplestats = lavsamplestats, lavdata = lavdata,
+            lavimplied = lavimplied, lavh1 = lavh1,
+            lavcache = lavcache, lavoptions = lavoptions)
+    } else if(information == "expected") {
+        I1 <- lav_model_h1_information_expected(lavmodel = lavmodel,
+            lavsamplestats = lavsamplestats, lavdata = lavdata,
+            lavimplied = lavimplied, lavh1 = lavh1,
+            lavcache = lavcache, lavoptions = lavoptions)
+    } else if(information == "first.order") {
+        I1 <- lav_model_h1_information_firstorder(lavmodel = lavmodel,
+            lavsamplestats = lavsamplestats, lavdata = lavdata,
+            lavimplied = lavimplied, lavh1 = lavh1,
+            lavcache = lavcache, lavoptions = lavoptions)
+    }
+
+    if(lavoptions$se %in% c("robust.huber.white", "robust.sem")) {
+        J1 <- lav_model_h1_information_firstorder(lavmodel = lavmodel,
+            lavsamplestats = lavsamplestats, lavdata = lavdata,
+            lavimplied = lavimplied, lavh1 = lavh1,
+            lavcache = lavcache, lavoptions = lavoptions)
+    }
+
+    # compute ACOV per group
+    ACOV <- vector("list", length = lavdata@ngroups)
+    for(g in 1:lavdata@ngroups) {
+
+        # denominator
+        if(lavdata@nlevels == 1L) {
+            Ng <- lavsamplestats@nobs[[g]]
+        } else {
+            Ng <- lavdata@Lp[[g]]$nclusters[[2]]
+        }
+
+        # invert information
+        I1.g.inv <- try(lav_matrix_symmetric_inverse(I1[[g]]), silent = TRUE)
+        if(inherits(I1.g.inv, "try-error")) {
+            stop("lavaan ERROR: could not invert h1 information matrix in group ", g)
+        }
+
+        # which type of se?
+        if(lavoptions$se %in% c("standard", "none")) {
+            ACOV[[g]] <- 1/Ng * I1.g.inv
+        } else if(lavoptions$se %in% c("robust.huber.white", "robust.sem")) {
+            ACOV[[g]] <- 1/Ng * (I1.g.inv %*% J1[[g]] %*% I1.g.inv)
+        }
+    }
+
+    ACOV
+}
+
+
