@@ -2,8 +2,9 @@
 
 # initial version: YR 25/03/2009: `methods' for the Model class
 # - YR 14 Jan 2014: rename object -> lavmodel, all functions as lav_model_*
+# - YR 20 Nov 2021: add lav_model_dmmdpar
 
-lav_model_get_parameters <- function(lavmodel = NULL, GLIST = NULL, 
+lav_model_get_parameters <- function(lavmodel = NULL, GLIST = NULL,
                                      type = "free", extra = TRUE) {
 
     # type == "free": only non-redundant free parameters (x)
@@ -25,7 +26,7 @@ lav_model_get_parameters <- function(lavmodel = NULL, GLIST = NULL,
         if(type == "free") {
             m.idx <- lavmodel@m.free.idx[[mm]]
             x.idx <- lavmodel@x.free.idx[[mm]]
-        #} else if(type == "unco") { 
+        #} else if(type == "unco") {
         #    m.idx <- lavmodel@m.unco.idx[[mm]]
         #    x.idx <- lavmodel@x.unco.idx[[mm]]
         } else if(type == "user") {
@@ -36,10 +37,10 @@ lav_model_get_parameters <- function(lavmodel = NULL, GLIST = NULL,
     }
 
     if(type == "user" && extra && sum(lavmodel@x.def.idx,
-                                      lavmodel@x.ceq.idx, 
+                                      lavmodel@x.ceq.idx,
                                       lavmodel@x.cin.idx) > 0L) {
         # we need 'free' x
-        x.free <- lav_model_get_parameters(lavmodel = lavmodel, GLIST = GLIST, 
+        x.free <- lav_model_get_parameters(lavmodel = lavmodel, GLIST = GLIST,
                                            type = "free")
         if(length(lavmodel@x.def.idx) > 0L) {
             x[lavmodel@x.def.idx] <- lavmodel@def.function(x.free)
@@ -75,7 +76,7 @@ lav_model_set_parameters <- function(lavmodel = NULL, x = NULL) {
 
                 if(lavmodel@estimator %in% c("WLS","DWLS","ULS","PML")) {
                     if(lavmodel@parameterization == "delta") {
-                        tmp[mm.in.group] <- 
+                        tmp[mm.in.group] <-
                         setResidualElements.LISREL(MLIST = tmp[mm.in.group],
                             num.idx = lavmodel@num.idx[[g]],
                             ov.y.dummy.ov.idx = lavmodel@ov.y.dummy.ov.idx[[g]],
@@ -106,7 +107,7 @@ lav_model_set_parameters <- function(lavmodel = NULL, x = NULL) {
 
 # create a standalone GLIST, filled with (new) x values
 # (avoiding a copy of lavmodel)
-lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL, 
+lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
                               type = "free", setDelta = TRUE,
                               m.el.idx = NULL, x.el.idx = NULL) {
 
@@ -118,6 +119,9 @@ lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
         if(type == "free") {
             M.EL.IDX <- lavmodel@m.free.idx[[mm]]
             X.EL.IDX <- lavmodel@x.free.idx[[mm]]
+        } else if(type == "unco") {
+            M.EL.IDX <- lavmodel@m.free.idx[[mm]]
+            X.EL.IDX <- lavmodel@x.unco.idx[[mm]]
         } else if(type == "full") {
             if(lavmodel@isSymmetric[mm]) {
                 N <- ncol(GLIST[[mm]])
@@ -144,7 +148,7 @@ lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
     }
 
     # theta parameterization: delta must be reset!
-    if(lavmodel@categorical && setDelta && 
+    if(lavmodel@categorical && setDelta &&
        lavmodel@parameterization == "theta") {
         nmat <- lavmodel@nmat
         for(g in 1:lavmodel@nblocks) {
@@ -159,6 +163,58 @@ lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
     GLIST
 }
 
+# derivative of model matrix (say, Psi, Theta) wrt the free elements
+# in that model matrix
+# returns a matrix with 0/1 entries
+# - rows are the nrow*ncol elements of the full matrix
+# - cols are the free parameters
+#
+# TOdo: use sparse matrices
+#
+lav_model_dmmdpar <- function(lavmodel, target = "theta", group = 1L) {
+
+    stopifnot(group <= lavmodel@ngroups)
+
+    # MLIST for this group
+    nmat <- lavmodel@nmat
+    # which mm belong to group g?
+    mm.in.group <- 1:nmat[group] + cumsum(c(0L,nmat))[group]
+    MLIST <- lavmodel@GLIST[ mm.in.group ]
+
+    # find target model matrix
+    mlist.idx <- which(names(MLIST) == target)
+    if(length(mlist.idx) == 0L) {
+        stop("lavaan ERROR: model matrix \"", target, "\" not found. Available model matrices are:\n  ", paste(names(MLIST), collapse = " "))
+    }
+
+    # target idx in GLIST
+    target.idx <- cumsum(c(0L, nmat))[group] + mlist.idx
+
+    # symmetric matrices (eg Psi, Theta)
+    if(lavmodel@isSymmetric[[target.idx]]) {
+        TARGET <- lavmodel@GLIST[[target.idx]]
+        P <- nrow(TARGET)
+
+        unique.idx <- unique(lavmodel@x.free.idx[[target.idx]])
+        row.idx <- match(lavmodel@x.free.idx[[target.idx]], unique.idx)
+        out <- matrix(0L, nrow = P*P, ncol = length(unique.idx))
+        IDX <- cbind(lavmodel@m.free.idx[[target.idx]], row.idx)
+        out[IDX] <- 1L
+
+    # non-symmetric matrices (eg Lambda, Beta)
+    } else {
+        TARGET <- lavmodel@GLIST[[target.idx]]
+        P <- nrow(TARGET); M <- ncol(TARGET)
+
+        row.idx <- seq_len(length(lavmodel@x.free.idx[[target.idx]]))
+        out <- matrix(0L, nrow = P*M, ncol = length(row.idx))
+        IDX <- cbind(lavmodel@m.free.idx[[target.idx]], row.idx)
+        out[IDX] <- 1L
+    }
+
+    out
+}
+
 # backwards compatibility
 # getModelParameters <- lav_model_get_parameters
 # setModelParameters <- lav_model_set_parameters
