@@ -15,7 +15,8 @@ lav_mvnorm_missing_h1_estimate_moments <- function(Y           = NULL,
                                                    Sinv.method = "eigen",
                                                    verbose     = FALSE,
                                                    max.iter    = 500L,
-                                                   tol         = 1e-05) {
+                                                   tol         = 1e-05,
+                                                   warn        = FALSE) {
 
     # check input
     Y <- as.matrix(Y); P <- NCOL(Y)
@@ -33,7 +34,18 @@ lav_mvnorm_missing_h1_estimate_moments <- function(Y           = NULL,
         Yp <- lav_samplestats_missing_patterns(Y = Y, Mp = Mp, wt = wt)
     }
 
+    if(is.null(max.iter)) {
+        max.iter <- 500L
+    }
+    if(is.null(tol)) {
+        tol <- 1e-05
+    }
+    if(is.null(warn)) {
+        warn <- FALSE
+    }
+
     # remove empty cases
+    N.full <- N
     if(length(Mp$empty.idx) > 0L) {
         if(!is.null(wt)) {
             N <- N - sum(wt[Mp$empty.idx])
@@ -50,13 +62,30 @@ lav_mvnorm_missing_h1_estimate_moments <- function(Y           = NULL,
 
     # starting values; zero covariances to guarantee a pd matrix
     if(!is.null(wt)) {
-        out <- stats::cov.wt(Y, wt = wt, method = "ML")
-        Mu0 <- out$center
-        var0 <- diag(out$cov)
+        tmp <- na.omit(cbind(wt, Y))
+        if(nrow(tmp) > 2L) {
+           Y.tmp  <- tmp[,-1, drop = FALSE]
+            wt.tmp <- tmp[,1]
+            out <- stats::cov.wt(Y.tmp, wt = wt.tmp, method = "ML")
+            Mu0 <- out$center
+            var0 <- diag(out$cov)
+        } else {
+            Mu0 <- base::.colMeans(Y, m = N.full, n = P, na.rm = TRUE)
+            Yc <- t( t(Y) - Mu0 )
+            var0 <- base::.colMeans(Yc*Yc, m = N.full, n = P, na.rm = TRUE)
+        }
     } else {
-        Mu0 <- base::.colMeans(Y, m = N, n = P, na.rm = TRUE)
+        Mu0 <- base::.colMeans(Y, m = N.full, n = P, na.rm = TRUE)
         Yc <- t( t(Y) - Mu0 )
-        var0 <- base::.colMeans(Yc*Yc, m = N, n = P, na.rm = TRUE)
+        var0 <- base::.colMeans(Yc*Yc, m = N.full, n = P, na.rm = TRUE)
+    }
+    # sanity check
+    bad.idx <- which(!is.finite(var0) | var0 == 0)
+    if(length(bad.idx) > 0L) {
+        var0[bad.idx] <- 1
+    }
+    bad.idx <- which(!is.finite(Mu0))
+    if(length(bad.idx) > 0L) {
     }
     Sigma0 <- diag(x = var0, nrow = P)
     Mu <- Mu0; Sigma <- Sigma0
@@ -89,8 +118,8 @@ lav_mvnorm_missing_h1_estimate_moments <- function(Y           = NULL,
 
         # check if Sigma is near-pd (+ poor fix)
         ev <- eigen(Sigma, symmetric = TRUE, only.values = TRUE)
-        tol <- 1e-6 # FIXME!
-        if(any(ev$values < tol)) {
+        evtol <- 1e-6 # FIXME!
+        if(any(ev$values < evtol)) {
             #too.small <- which( ev$values < tol )
             #ev$values[too.small] <- tol
             #ev$values <- ev$values + tol
@@ -141,6 +170,26 @@ lav_mvnorm_missing_h1_estimate_moments <- function(Y           = NULL,
                                                     minus.two = TRUE)/N
     }
 
+    # warning?
+    if(warn && i == max.iter) {
+        txt <- c("Maximum number of iterations reached when ",
+                 "computing the sample moments using EM; ",
+                 "use the em.h1.iter.max= argument to increase the number of ",
+                 "iterations")
+        warning(lav_txt2message(txt))
+    }
+
+    if(warn) {
+        ev <- eigen(Sigma, symmetric = TRUE, only.values = TRUE)$values
+        if(any(ev < 1e-05)) { # make an option?
+            txt <- c("The smallest eigenvalue of the EM estimated ",
+                     "variance-covariance matrix (Sigma) is smaller than ",
+                     "1e-05; this may cause numerical instabilities; ",
+                     "interpret the results with caution.")
+            warning(lav_txt2message(txt))
+        }
+    }
+
     list(Sigma = Sigma, Mu = Mu, fx = fx)
 }
 
