@@ -105,9 +105,7 @@ lav_options_default <- function(mimic = "lavaan") {
                 sample.cov.rescale = "default",
                 sample.icov        = TRUE,
                 ridge              = FALSE,
-                ridge.x            = FALSE,
                 ridge.constant     = "default",
-                ridge.constant.x   = 1e-5,
 
                 # multiple groups
                 group.label        = NULL,
@@ -179,7 +177,7 @@ lav_options_default <- function(mimic = "lavaan") {
 
                 # parallel
                 parallel               = "no",
-                ncpus                  = 1L,
+                ncpus                  = parallel::detectCores() - 1L,
                 cl                     = NULL,
                 iseed                  = NULL,
 
@@ -268,7 +266,7 @@ lav_options_set <- function(opt = NULL) {
         opt$group.equal <- character(0)
     } else if(is.null(opt$group.equal) || all(nchar(opt$group.equal) == 0L)) {
         if(opt$mimic == "Mplus") {
-            if(opt$categorical) {
+            if(opt$.categorical) {
                 opt$group.equal <- c("loadings", "thresholds")
             } else {
                 if(is.logical(opt$meanstructure) && !opt$meanstructure) {
@@ -308,10 +306,10 @@ lav_options_set <- function(opt = NULL) {
 
     # if categorical, and group.equal contains "intercepts", also add
     # thresholds (and vice versa)
-    if(opt$categorical && "intercepts" %in% opt$group.equal) {
+    if(opt$.categorical && "intercepts" %in% opt$group.equal) {
         opt$group.equal <- unique(c(opt$group.equal, "thresholds"))
     }
-    if(opt$categorical && "thresholds" %in% opt$group.equal) {
+    if(opt$.categorical && "thresholds" %in% opt$group.equal) {
         opt$group.equal <- unique(c(opt$group.equal, "intercepts"))
     }
 
@@ -336,12 +334,8 @@ lav_options_set <- function(opt = NULL) {
 
     # clustered
     # brute-force override (for now)
-    if(opt$clustered && !opt$multilevel) {
+    if(opt$.clustered && !opt$.multilevel) {
         opt$meanstructure <- TRUE
-        #opt$missing <- "listwise"
-        #if(opt$missing == "ml") {
-        #    opt$optim.gradient = "numerical"
-        #}
 
         if(opt$estimator == "mlr") {
             opt$estimator <- "ml"
@@ -351,6 +345,9 @@ lav_options_set <- function(opt = NULL) {
             opt$estimator <- "ml"
             opt$test <- "satorra.bentler"
             opt$se <- "robust.cluster.sem"
+        } else if(opt$.categorical) {
+            opt$test <- "satorra.bentler"
+            opt$se <- "robust.cluster.sem"
         }
 
         # test
@@ -380,7 +377,7 @@ lav_options_set <- function(opt = NULL) {
 
         # information
         if(opt$information[1] == "default") {
-            if(opt$se == "robust.cluster") {
+            if(opt$se == "robust.cluster" && opt$estimator == "ml") {
                 opt$information[1] <- "observed"
             } else {
                 opt$information[1] <- "expected"
@@ -397,33 +394,25 @@ lav_options_set <- function(opt = NULL) {
 
     # multilevel
     # brute-force override (for now)
-    if(opt$multilevel) {
+    if(opt$.multilevel) {
         opt$meanstructure <- TRUE
-        #opt$missing <- "listwise" # still needed for 0.6-8 (otherwise, we
-        #                          # we break tidySEM tests where they set
-        #                          # missing = "fiml" + multilevel
-        #if(opt$missing %in% c("ml", "fiml", "direct")) {
-            #opt$optim.gradient = "numerical"
-            #opt$optim.gradient = "complex"
-        #}
 
         # test
         if(length(opt$test) == 1L && opt$test == "default") {
-            opt$test <- "standard"
+            # ok, will be set later
         } else if(all(opt$test %in% c("none", "standard","yuan.bentler"))) {
             # nothing to do
-        } else if(opt$se == "robust") {
-            opt$test <- "yuan.bentler"
         } else {
             stop("lavaan ERROR: `test' argument must one of \"none\", \"standard\" or \"yuan.bentler\" in the multilevel case")
         }
 
         # se
         if(opt$se == "default") {
-            opt$se <- "standard"
-        } else if(opt$se %in% c("none", "standard", "robust.huber.white", "sandwich")) {
+            # ok, will be set later
+        } else if(opt$se %in% c("none", "standard", "robust.huber.white",
+                                "sandwich")) {
             # nothing to do
-        } else if(opt$se == "robust.sem") {
+        } else if(opt$se == "robust") {
             opt$se <- "robust.huber.white"
         } else {
             stop("lavaan ERROR: `se' argument must one of \"none\", \"standard\" or \"robust.huber.white\" in the multilevel case")
@@ -438,10 +427,9 @@ lav_options_set <- function(opt = NULL) {
         }
     }
 
-
     # missing
     if(opt$missing == "default") {
-        if(opt$mimic == "Mplus" && !opt$categorical &&
+        if(opt$mimic == "Mplus" && !opt$.categorical &&
            opt$estimator %in% c("default", "ml", "mlr")) {
             # since version 5?
             opt$missing <- "ml"
@@ -450,8 +438,8 @@ lav_options_set <- function(opt = NULL) {
             opt$missing <- "listwise"
         }
     } else if(opt$missing %in% c("ml", "direct", "fiml")) {
-        #if(opt$categorical && opt$estimator != "mml") {
-        if(opt$categorical) {
+        #if(opt$.categorical && opt$estimator != "mml") {
+        if(opt$.categorical) {
             stop("lavaan ERROR: missing = ", dQuote(opt$missing),
                  " not available in the categorical setting")
         }
@@ -462,8 +450,8 @@ lav_options_set <- function(opt = NULL) {
                  dQuote(opt$estimator))
         }
     } else if(opt$missing %in% c("ml.x", "direct.x", "fiml.x")) {
-        #if(opt$categorical && opt$estimator != "mml") {
-        if(opt$categorical) {
+        #if(opt$.categorical && opt$estimator != "mml") {
+        if(opt$.categorical) {
             stop("lavaan ERROR: missing = ", dQuote(opt$missing),
                  " not available in the categorical setting")
         }
@@ -476,7 +464,7 @@ lav_options_set <- function(opt = NULL) {
     } else if(opt$missing %in% c("two.stage", "twostage", "two-stage",
                                  "two.step",  "twostep",  "two-step")) {
         opt$missing <- "two.stage"
-        if(opt$categorical) {
+        if(opt$.categorical) {
             stop("lavaan ERROR: missing=\"two.stage\" not available in the categorical setting")
         }
         if(opt$estimator %in% c("mlm", "mlmv", "gls", "wls", "wlsm", "wlsmv",
@@ -488,7 +476,7 @@ lav_options_set <- function(opt = NULL) {
                                  "robust.two.step",  "robust.twostep",
                                  "robust-two-step")) {
         opt$missing <- "robust.two.stage"
-        if(opt$categorical) {
+        if(opt$.categorical) {
             stop("lavaan ERROR: missing=\"robust.two.stage\" not available in the categorical setting")
         }
         if(opt$estimator %in% c("mlm", "mlmv", "gls", "wls", "wlsm", "wlsmv",
@@ -679,7 +667,7 @@ lav_options_set <- function(opt = NULL) {
 
     # default estimator
     if(opt$estimator == "default") {
-        if(opt$categorical) {
+        if(opt$.categorical) {
             opt$estimator <- "wlsmv"
         } else {
             opt$estimator <- "ml"
@@ -1168,7 +1156,7 @@ lav_options_set <- function(opt = NULL) {
 
 
     # special stuff for categorical
-    if(opt$categorical) {
+    if(opt$.categorical) {
         opt$meanstructure <- TRUE # Mplus style
         if(opt$estimator == "ML") {
             stop("lavaan ERROR: estimator ML for ordered data is not supported yet. Use WLSMV instead.")
@@ -1409,7 +1397,7 @@ lav_options_set <- function(opt = NULL) {
         if(opt$estimator == "ML" && (opt$mimic == "Mplus" ||
                                      opt$mimic == "lavaan")) {
             opt$conditional.x <- FALSE
-        } else if(opt$categorical) {
+        } else if(opt$.categorical) {
             opt$conditional.x <- TRUE
         } else {
             opt$conditional.x <- FALSE
@@ -1425,12 +1413,18 @@ lav_options_set <- function(opt = NULL) {
 
     # fixed.x
     if(is.logical(opt$fixed.x)) {
+        #if(opt$conditional.x && opt$fixed.x == FALSE && !opt$.multilevel) {
         if(opt$conditional.x && opt$fixed.x == FALSE) {
             stop("lavaan ERROR: fixed.x = FALSE is not supported when conditional.x = TRUE.")
         }
+        if(opt$fixed.x && is.character(opt$start) && opt$start == "simple") {
+            warning("lavaan WARNING: start = \"simple\" implies fixed.x = FALSE")
+            opt$fixed.x <- FALSE
+        }
     } else if(opt$fixed.x == "default") {
-        if(opt$estimator %in% c("MML", "ML") && (opt$mimic == "Mplus" ||
-                                     opt$mimic == "lavaan")) {
+        if(opt$estimator %in% c("MML", "ML") &&
+           (opt$mimic == "Mplus" || opt$mimic == "lavaan") &&
+           is.character(opt$start) && opt$start != "simple") { # new in 0.6-12
             opt$fixed.x <- TRUE
         } else if(opt$conditional.x) {
             opt$fixed.x <- TRUE
@@ -1441,7 +1435,6 @@ lav_options_set <- function(opt = NULL) {
         stop("lavaan ERROR: fixed.x must be TRUE, FALSE or \"default\"\n")
     }
 
-
     # meanstructure again
     if(opt$missing %in% c("ml", "ml.x") || opt$model.type == "growth") {
         opt$meanstructure <- TRUE
@@ -1733,10 +1726,13 @@ lav_options_set <- function(opt = NULL) {
 
 
     # group.w.free
-    #if(opt$group.w.free && opt$categorical) {
+    #if(opt$group.w.free && opt$.categorical) {
     #    stop("lavaan ERROR: group.w.free = TRUE is not supported (yet) in the categorical setting.")
     #}
 
+    # in order not to break semTools and blavaan, we restore categorical:
+    opt$categorical <- opt$.categorical
+
     if(opt$debug) { cat("lavaan DEBUG: lavaanOptions OUT\n"); str(opt) }
 
     opt
