@@ -1,12 +1,13 @@
 
-# what are the block values (not necessarly 1..nb)
+# what are the block values (not necessarily integers)
 lav_partable_block_values <- function(partable) {
 
     if(is.null(partable$block)) {
         block.values <- 1L
     } else {
         # always integers
-        tmp <- partable$block[ partable$block > 0L ] # non-zero only
+        tmp <- partable$block[  partable$block > 0L &       # non-zero only
+                               !partable$op %in% c("==", "<", ">", ":=") ]
         block.values <- unique(na.omit(tmp)) # could be, eg, '2' only
     }
 
@@ -21,13 +22,23 @@ lav_partable_nblocks <- function(partable) {
 # what are the group values (not necessarily integers)
 lav_partable_group_values <- function(partable) {
 
-    if(is.null(partable$group)) {
+    # FLAT?
+    if(any(partable$op == ":")) {
+        colon.idx <- which(partable$op == ":" &
+                           tolower(partable$lhs) == "group")
+        if(length(colon.idx) > 0L) {
+            group.values <- unique(partable$rhs[colon.idx])
+        }
+    # regular partable
+    } else if(is.null(partable$group)) {
         group.values <- 1L
     } else if(is.numeric(partable$group)) {
-        tmp <- partable$group[ partable$group > 0L ]
+        tmp <- partable$group[ partable$group > 0L &
+                              !partable$op %in% c("==", "<", ">", ":=") ]
         group.values <- unique(na.omit(tmp))
     } else { # character
-        tmp <- partable$group[nchar(partable$group) > 0L]
+        tmp <- partable$group[ nchar(partable$group) > 0L &
+                              !partable$op %in% c("==", "<", ">", ":=") ]
         group.values <- unique(na.omit(tmp))
     }
 
@@ -42,13 +53,25 @@ lav_partable_ngroups <- function(partable) {
 # what are the level values (not necessarily integers)
 lav_partable_level_values <- function(partable) {
 
-    if(is.null(partable$level)) {
+
+    # FLAT?
+    if(any(partable$op == ":")) {
+        colon.idx <- which(partable$op == ":" &
+                           tolower(partable$lhs) == "level")
+        level.values <- integer(0L)
+        if(length(colon.idx) > 0L) {
+            level.values <- unique(partable$rhs[colon.idx])
+        }
+    # regular partable
+    } else if(is.null(partable$level)) {
         level.values <- 1L
     } else if(is.numeric(partable$level)) {
-        tmp <- partable$level[ partable$level > 0L ]
+        tmp <- partable$level[  partable$level > 0L &
+                               !partable$op %in% c("==", "<", ">", ":=") ]
         level.values <- unique(na.omit(tmp))
     } else { # character
-        tmp <- partable$level[nchar(partable$level) > 0L]
+        tmp <- partable$level[ nchar(partable$level) > 0L &
+                              !partable$op %in% c("==", "<", ">", ":=") ]
         level.values <- unique(na.omit(tmp))
     }
 
@@ -67,7 +90,8 @@ lav_partable_efa_values <- function(partable) {
         efa.values <- character(0L)
     } else { # should be character
         EFA <- as.character(partable$efa)
-        tmp <- EFA[nchar(EFA) > 0L]
+        tmp <- EFA[ nchar(EFA) > 0L &
+                   !partable$op %in% c("==", "<", ">", ":=") ]
         efa.values <- unique(na.omit(tmp))
     }
 
@@ -122,7 +146,7 @@ lav_partable_ndat <- function(partable) {
 
                 # except within-only 'y'
                 ov.names.y <- lav_partable_vnames(partable, "ov.nox", block = b)
-                ov.names.y2 <- unlist(lav_partable_vnames(partable, "ov.nox",
+                ov.names.y2 <- unlist(lav_partable_vnames(partable, "ov",
                                       block = seq_len(nblocks)[-b]))
                 ov.names.y <- ov.names.y[ !ov.names.y %in% ov.names.y2 ]
                 if(length(ov.names.y) > 0L) {
@@ -131,7 +155,7 @@ lav_partable_ndat <- function(partable) {
 
                 # except within-only 'x' (unless fixed.x)
                 ov.names.x <- lav_partable_vnames(partable, "ov.x", block = b)
-                ov.names.x2 <- unlist(lav_partable_vnames(partable, "ov.x",
+                ov.names.x2 <- unlist(lav_partable_vnames(partable, "ov",
                                       block = seq_len(nblocks)[-b]))
                 ov.names.x <- ov.names.x[ !ov.names.x %in% ov.names.x2 ]
                 if(!fixed.x && length(ov.names.x) > 0L) {
@@ -148,7 +172,7 @@ lav_partable_ndat <- function(partable) {
             nvar.x <- length(ov.names.x)
             pstar.x <- nvar.x * (nvar.x + 1) / 2
             if(meanstructure) {
-                if(nlevels > 1L && b == 1L) {
+                if(nlevels > 1L && (b %% nlevels) == 1L) {
                     # do nothing, they are already removed
                 } else {
                     pstar.x <- pstar.x + nvar.x
@@ -201,7 +225,7 @@ lav_partable_ndat <- function(partable) {
     sum(ndat)
 }
 
-# total number of free parameters
+# total number of free parameters (ignoring equality constraints)
 lav_partable_npar <- function(partable) {
 
     # we only assume non-zero values
@@ -211,6 +235,9 @@ lav_partable_npar <- function(partable) {
 
 # global degrees of freedom: ndat - npar
 # ignoring constraints! (not very useful)
+#
+# we need to find the rank of con.jac to find the exact amount
+# of non-redundant equality constraints (this is done in lav_test.R)
 lav_partable_df <- function(partable) {
 
     npar <- lav_partable_npar(partable)
@@ -243,6 +270,11 @@ lav_partable_covariance_reorder <- function(partable,
     }
     if(is.null(lv.names)) {
         lv.names <- lav_partable_vnames(partable, "lv")
+        # add random slopes (if any)
+        if( !is.null(partable$rv) && any(nchar(partable$rv) > 0L) ) {
+            RV.names <- unique(partable$rv[nchar(partable$rv) > 0L])
+            lv.names <- c(lv.names, RV.names)
+        }
     } else {
         lv.names <- unlist(lv.names)
     }
@@ -267,46 +299,48 @@ lav_partable_covariance_reorder <- function(partable,
 }
 
 # add a single parameter to an existing parameter table
-lav_partable_add <- function(PT, add = list()) {
+lav_partable_add <- function(partable = NULL, add = list()) {
 
-    # treat PT as list, not as a data.frame
-    PT <- as.list(PT)
+    # treat partable as list, not as a data.frame
+    partable <- as.list(partable)
 
     # number of elements
-    nel <- length(PT$lhs)
+    nel <- length(partable$lhs)
 
     # add copy of last row
-    for(c in seq_len(length(PT))) {
-        if(is.integer(PT[[c]][[1]])) {
-            if(PT[[c]][nel] == 0L) {
-                PT[[c]][nel + 1] <- 0L
-            } else if(PT[[c]][nel] == 1L) {
-                PT[[c]][nel + 1] <- 1L
+    for(c in seq_len(length(partable))) {
+        if(is.integer(partable[[c]][[1]])) {
+            if(partable[[c]][nel] == 0L) {
+                partable[[c]][nel + 1] <- 0L
+            } else if(partable[[c]][nel] == 1L) {
+                partable[[c]][nel + 1] <- 1L
             } else {
-                PT[[c]][nel + 1] <- PT[[c]][nel] + 1L
+                partable[[c]][nel + 1] <- partable[[c]][nel] + 1L
             }
-        } else if(is.character(PT[[c]][[1]])) {
-            PT[[c]][nel + 1] <- ""
-        } else if(is.numeric(PT[[c]][[1]])) {
-            PT[[c]][nel + 1] <- 0
+        } else if(is.character(partable[[c]][[1]])) {
+            partable[[c]][nel + 1] <- ""
+        } else if(is.numeric(partable[[c]][[1]])) {
+            partable[[c]][nel + 1] <- 0
         } else {
-            PT[[c]][nel + 1] <- PT[[c]][nel]
+            partable[[c]][nel + 1] <- partable[[c]][nel]
         }
 
         # replace
-        if(names(PT)[c] %in% names(add)) {
-            PT[[c]][nel + 1] <- add[[ names(PT)[c] ]]
+        if(names(partable)[c] %in% names(add)) {
+            partable[[c]][nel + 1] <- add[[ names(partable)[c] ]]
         }
 
     }
 
-    PT
+    partable
 }
 
 
-# look for row idx of p1 elements in p2
+# look for p2-row-idx of p1 elements
+# p1 is usually a subset of p2
 # return NA if not found
-lav_partable_map_id_p1_in_p2 <- function(p1, p2, stopifnotfound = TRUE) {
+lav_partable_map_id_p1_in_p2 <- function(p1, p2, stopifnotfound = TRUE,
+                                         exclude.nonpar = TRUE) {
 
     # check if we have a 'block' column (in both p1 and p2)
     if(is.null(p1$block)) {
@@ -324,9 +358,16 @@ lav_partable_map_id_p1_in_p2 <- function(p1, p2, stopifnotfound = TRUE) {
         }
     }
 
-    # get all parameters that have a '.p*' plabel
-    # (they exclude "==", "<", ">", ":=")
-    p1.idx <- which(grepl("\\.p", p1$plabel))
+    # ALL rows from p1, or only 'parameters'?
+    if(exclude.nonpar) {
+        # get all parameters that have a '.p*' plabel
+        # (they exclude "==", "<", ">", ":=")
+        p1.idx <- which(grepl("\\.p", p1$plabel))
+    } else {
+        # all of it
+        # note: block should be '0' in both p1 and p2
+        p1.idx <- seq_len( length(p1$lhs) )
+    }
     np1 <- length(p1.idx)
 
     # return p2.id
