@@ -323,7 +323,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     # handle lower/upper boundary points
     if(!is.null(x$lower)) {
         b.idx <- which( abs(x$lower - x$est) < sqrt(.Machine$double.eps) &
-                        is.finite(x$se) & x$se != 0.0)
+                        (is.na(x$se) | (is.finite(x$se) & x$se != 0.0)) )
         if(length(b.idx) > 0L && !is.null(x$pvalue)) {
             m[b.idx, "pvalue"] <- ""
             if(is.null(x$label)) {
@@ -777,7 +777,8 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
         cat("\n")
         #print.lavaan.parameterEstimates(y$PE, ..., nd = nd)
 
-        lav_object_print_short_summary(y$STRUC.FIT)
+        print(summary(y$STRUC.FIT, fit.measures = FALSE, estimates = FALSE,
+                                    modindices = FALSE))
         FIT <- fitMeasures(y$STRUC.FIT, fit.measures="default")
         if(FIT["df"] > 0) {
             print.lavaan.fitMeasures( FIT, add.h0 = FALSE )
@@ -849,3 +850,324 @@ lav_txt2message <- function(txt, header = "lavaan WARNING:",
     out
 }
 
+# new in 0.6-12
+print.lavaan.summary <- function(x, ..., nd = 3L) {
+
+    y <- unclass(x) # change to ordinary list
+
+    # get nd, if it is stored as an attribute
+    ND <- attr(y, "nd")
+    if(!is.null(ND) && is.numeric(ND)) {
+        nd <- as.integer(ND)
+    }
+
+    # header
+    if(!is.null(y$header)) {
+        lavaan.version   <- y$header$lavaan.version
+        sam.approach     <- y$header$sam.approach
+        optim.method     <- y$header$optim.method
+        optim.iterations <- y$header$optim.iterations
+        optim.converged  <- y$header$optim.converged
+
+        # sam or sem?
+        if(sam.approach) {
+            cat("This is ",
+                sprintf("lavaan %s", lavaan.version),
+                " -- using the SAM approach to SEM\n", sep = "")
+        } else {
+            cat(sprintf("lavaan %s ", lavaan.version))
+
+            # Convergence or not?
+            if(optim.method == "none") {
+                cat("-- DRY RUN with 0 iterations --\n")
+            } else if(optim.iterations > 0) {
+                if(optim.converged) {
+                    cat(sprintf("ended normally after %i iterations\n",
+                        optim.iterations))
+                } else {
+                    cat(sprintf("did NOT end normally after %i iterations\n",
+                        optim.iterations))
+                    cat("** WARNING ** Estimates below are most likely unreliable\n")
+                }
+            } else {
+                cat("did not run (perhaps do.fit = FALSE)?\n")
+                cat("** WARNING ** Estimates below are simply the starting values\n")
+            }
+        }
+    }
+
+    # optim
+    if(!is.null(y$optim)) {
+        estimator      <- y$optim$estimator
+        estimator.args <- y$optim$estimator.args
+        optim.method   <- y$optim$optim.method
+        npar           <- y$optim$npar
+        eq.constraints <- y$optim$eq.constraints
+        nrow.ceq.jac   <- y$optim$nrow.ceq.jac
+        nrow.cin.jac   <- y$optim$nrow.cin.jac
+        nrow.con.jac   <- y$optim$nrow.con.jac
+        con.jac.rank   <- y$optim$con.jac.rank
+
+        cat("\n")
+        # cat("Optimization information:\n\n")
+
+        c1 <- c("Estimator")
+        # second column
+        tmp.est <- toupper(estimator)
+        if(tmp.est == "DLS") {
+            dls.first.letter <- substr(estimator.args$dls.GammaNT,
+                                       1L, 1L)
+            tmp.est <- paste("DLS-", toupper(dls.first.letter), sep = "")
+        }
+        c2 <- tmp.est
+
+        # additional estimator args
+        if(!is.null(estimator.args) &&
+           length(estimator.args) > 0L) {
+            if(estimator == "DLS") {
+                c1 <- c(c1, "Estimator DLS value for a")
+                c2 <- c(c2, estimator.args$dls.a)
+            }
+        }
+
+        # optimization method + npar
+        c1 <- c(c1, "Optimization method", "Number of model parameters")
+        c2 <- c(c2, toupper(optim.method), npar)
+
+        # optional output
+        if(eq.constraints) {
+            c1 <- c(c1, "Number of equality constraints")
+            c2 <- c(c2, nrow.ceq.jac)
+        }
+        if(nrow.cin.jac > 0L) {
+            c1 <- c(c1, "Number of inequality constraints")
+            c2 <- c(c2, nrow.cin.jac)
+        }
+        if(nrow.con.jac > 0L) {
+            if(con.jac.rank == (nrow.ceq.jac + nrow.cin.jac)) {
+            # nothing to do (don't print, as this is redundant information)
+            } else {
+                c1 <- c(c1, "Row rank of the constraints matrix")
+                c2 <- c(c2, con.jac.rank)
+            }
+        }
+
+        # format
+        c1 <- format(c1, width = 40L)
+        c2 <- format(c2, width = 11L + max(0, (nd - 3L)) * 4L,
+                     justify = "right")
+
+        # character matrix
+        M <- cbind(c1, c2, deparse.level = 0)
+        colnames(M) <- rep("",  ncol(M))
+        rownames(M) <- rep(" ", nrow(M))
+
+        # print
+        write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+    }
+
+    # sam header
+    if(!is.null(y$sam.header)) {
+        cat("\n")
+        sam.method          <- y$sam.header$sam.method
+        sam.local.options   <- y$sam.header$sam.local.options
+        sam.mm.list         <- y$sam.header$sam.mm.list
+        sam.mm.estimator    <- y$sam.header$sam.mm.estimator
+        sam.struc.estimator <- y$sam.header$sam.struc.estimator
+
+        # sam method
+        c1 <- c("SAM method")
+        c2 <- toupper(sam.method)
+
+        # options
+        if(sam.method == "local") {
+            c1 <- c(c1, "Mapping matrix M method")
+            c2 <- c(c2, sam.local.options$M.method)
+            # TODo: more!
+        }
+
+        # number of measurement blocks
+        c1 <- c(c1, "Number of measurement blocks")
+        c2 <- c(c2, length(sam.mm.list))
+
+        # estimator measurement blocks
+        c1 <- c(c1, "Estimator measurement part")
+        c2 <- c(c2, sam.mm.estimator)
+
+        # estimator structural part
+        c1 <- c(c1, "Estimator  structural part")
+        c2 <- c(c2, sam.struc.estimator)
+
+        # format
+        c1 <- format(c1, width = 40L)
+        c2 <- format(c2, width = 11L + max(0, (nd - 3L)) * 4L,
+                     justify = "right")
+
+        # character matrix
+        M <- cbind(c1, c2, deparse.level = 0)
+        colnames(M) <- rep("",  ncol(M))
+        rownames(M) <- rep(" ", nrow(M))
+
+        # print
+        write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+    }
+
+    # efa/rotation
+    if(!is.null(y$rotation)) {
+        cat("\n")
+        rotation      <- y$rotation
+        rotation.args <- y$rotation.args
+
+        #cat("Rotation information:\n\n")
+        # container
+        c1 <- c2 <- character(0L)
+
+        # rotation method
+        c1 <- c(c1, "Rotation method")
+        if(rotation$rotation == "none") {
+            MM <- toupper(rotation$rotation)
+        } else if(rotation$rotation.args$orthogonal) {
+            MM <- paste(toupper(rotation$rotation), " ", "ORTHOGONAL",
+                    sep = "")
+        } else {
+            MM <- paste(toupper(rotation$rotation), " ", "OBLIQUE",
+                        sep = "")
+        }
+        c2 <- c(c2, MM)
+
+        if(rotation$rotation != "none") {
+
+            # method options
+            if(rotation$rotation == "geomin") {
+                c1 <- c(c1, "Geomin epsilon")
+                c2 <- c(c2, rotation$rotation.args$geomin.epsilon)
+            } else if(rotation$rotation == "orthomax") {
+                c1 <- c(c1, "Orthomax gamma")
+                c2 <- c(c2, rotation$rotation.args$orthomax.gamma)
+            } else if(rotation$rotation == "cf") {
+                c1 <- c(c1, "Crawford-Ferguson gamma")
+                c2 <- c(c2, rotation$rotation.args$cf.gamma)
+            } else if(rotation$rotation == "oblimin") {
+                c1 <- c(c1, "Oblimin gamma")
+                c2 <- c(c2, rotation$rotation.args$oblimin.gamma)
+            }
+
+            # rotation algorithm
+            c1 <- c(c1, "Rotation algorithm (rstarts)")
+            tmp <- paste(toupper(rotation$rotation.args$algorithm),
+                         " (", rotation$rotation.args$rstarts, ")", sep = "")
+            c2 <- c(c2, tmp)
+
+            # Standardized metric (or not)
+            c1 <- c(c1, "Standardized metric")
+            if(rotation$rotation.args$std.ov) {
+                c2 <- c(c2, "TRUE")
+            } else {
+                c2 <- c(c2, "FALSE")
+            }
+
+            # Row weights
+            c1 <- c(c1, "Row weights")
+            tmp.txt <- rotation$rotation.args$row.weights
+            c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                              substring(tmp.txt, 2), sep = ""))
+        }
+
+        # format c1/c2
+        c1 <- format(c1, width = 33L)
+        c2 <- format(c2, width = 18L + max(0, (nd - 3L)) * 4L,
+                     justify = "right")
+
+        # create character matrix
+        M <- cbind(c1, c2, deparse.level = 0)
+        colnames(M) <- rep("",  ncol(M))
+        rownames(M) <- rep(" ", nrow(M))
+
+        # print
+        write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+    }
+
+    # data object
+    if(!is.null(y$data)) {
+        cat("\n")
+        lav_data_print_short(y$data, nd = nd)
+    }
+
+    # sam local stats: measurement blocks + structural part
+    if(!is.null(y$sam)) {
+        cat("\n")
+        sam.method    <- y$sam$sam.method
+        sam.mm.table  <- y$sam$sam.mm.table
+        sam.mm.rel    <- y$sam$sam.mm.rel
+        sam.struc.fit <- y$sam$sam.struc.fit
+        ngroups       <- y$sam$ngroups
+        group.label   <- y$sam$group.label
+
+        # measurement
+        tmp <- sam.mm.table
+        if(sam.method == "global") {
+            cat("Summary Information Measurement Part:\n\n")
+        } else {
+            cat("Summary Information Measurement + Structural:\n\n")
+        }
+        print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+
+        if(sam.method == "local") {
+            # reliability information
+            c1 <- c2 <- character(0L)
+            if(ngroups == 1L) {
+                cat("\n")
+                cat("  Model-based reliability latent variables:\n\n")
+                tmp <- data.frame(as.list(sam.mm.rel[[1]]))
+                class(tmp) <- c("lavaan.data.frame", "data.frame")
+                print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+            } else {
+                cat("\n")
+                cat("  Model-based reliability latent variables (per group):\n")
+                for(g in 1:ngroups) {
+                    cat("\n")
+                    cat("  Group ", g, " [", group.label[g], "]:\n\n",
+                        sep = "")
+                    tmp <- data.frame(as.list(sam.mm.rel[[g]]))
+                    class(tmp) <- c("lavaan.data.frame", "data.frame")
+                    print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+                }
+            }
+
+            cat("\n")
+            cat("  Summary Information Structural part:\n\n")
+            tmp <- data.frame(as.list(sam.struc.fit))
+            class(tmp) <- c("lavaan.data.frame", "data.frame")
+            print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+        }
+    }
+
+    # test statistics
+    if(!is.null(y$test)) {
+        cat("\n")
+        lav_test_print(y$test, nd = nd)
+    }
+
+    # extra fit measures (if present)
+    if(!is.null(y$fit)) {
+        print.lavaan.fitMeasures(y$fit, nd = nd, add.h0 = FALSE )
+    }
+
+    # parameter table
+    if(!is.null(y$pe)) {
+        PE <- y$pe
+        class(PE) <- c("lavaan.parameterEstimates", "lavaan.data.frame",
+                       "data.frame")
+        print(PE, nd = nd)
+    }
+
+    # modification indices
+    if(!is.null(y$mi)) {
+        cat("Modification Indices:\n\n")
+        MI <- y$mi
+        rownames(MI) <- NULL
+        print(MI, nd = nd)
+    }
+
+    invisible(y)
+}
