@@ -22,11 +22,10 @@ lavTestLRT <- function(object, ..., method = "default", A.method = "delta",
 
     mcall <- match.call(expand.dots = TRUE)
     dots <- list(...)
-
     modp <- if(length(dots)) {
-        sapply(dots, is, "lavaan")
+        sapply(dots, inherits, "lavaan")
     } else {
-         logical(0)
+         logical(0L)
     }
 
     # some general properties (taken from the first model)
@@ -50,7 +49,7 @@ lavTestLRT <- function(object, ..., method = "default", A.method = "delta",
         names(mods) <- model.names
     } else {
         names(mods) <- sapply(as.list(mcall)[which(c(FALSE, TRUE, modp))],
-                              deparse)
+                              function(x) deparse(x))
     }
 
     ## put them in order (using number of free parameters)
@@ -188,10 +187,13 @@ lavTestLRT <- function(object, ..., method = "default", A.method = "delta",
     Df.delta     <- c(NA, diff(Df))
 
     # check for negative values in STAT.delta
-    if(any(STAT.delta[-1] < 0)) {
-        warning("lavaan WARNING: some restricted models fit better than less ",
-                "\n\t restricted models; either these models are not nested, or",
-                "\n\t the less restricted model failed to reach a global optimum.")
+    # but with a tolerance (0.6-12)!
+    if(any(STAT.delta[-1] < -1*.Machine$double.eps^(1/3))) {
+        txt <- c("Some restricted models fit better than less ",
+                 "restricted models; either these models are not nested, or",
+                 "the less restricted model failed to reach a global optimum.")
+        txt <- c(txt, " Smallest difference = ", min(STAT.delta[-1]))
+        warning(lav_txt2message(txt))
     }
 
     # correction for scaled test statistics
