@@ -57,8 +57,13 @@ lav_test_diff_Satorra2000 <- function(m1, m0, H1 = TRUE, A.method = "delta",
         if(is.null(A)) {
             A <- lav_test_diff_A(m1, m0, method = A.method, reference = "H1")
             # take into account equality constraints m1
-            if(A.method == "delta" && m1@Model@eq.constraints) {
-                A <- A %*% t(m1@Model@eq.constraints.K)
+            if(A.method == "delta") {
+                if(m1@Model@eq.constraints) {
+                    A <- A %*% t(m1@Model@eq.constraints.K)
+                } else if(.hasSlot(m1@Model, "ceq.simple.only") &&
+                          m1@Model@ceq.simple.only) {
+                    A <- A %*% t(m1@Model@ceq.simple.K)
+                }
             }
             if(debug) print(A)
         }
@@ -82,6 +87,9 @@ lav_test_diff_Satorra2000 <- function(m1, m0, H1 = TRUE, A.method = "delta",
             # take into account equality constraints m1
             if(m0@Model@eq.constraints) {
                 A <- A %*% t(m0@Model@eq.constraints.K)
+            } else if(.hasSlot(m0@Model, "ceq.simple.only") &&
+                      m0@Model@ceq.simple.only) {
+                A <- A %*% t(m0@Model@ceq.simple.K)
             }
             if(debug) print(A)
         }
@@ -226,14 +234,14 @@ lav_test_diff_SatorraBentler2010 <- function(m1, m0, H1 = FALSE) {
                          symmetric=TRUE, only.values=TRUE)$values
         if(any(eigvals < -1 * .Machine$double.eps^(3/4))) {
             warning(
-  "lavaan WARNING: information matrix of the M01 model is not positive definite.\n",
-"                  As a result, the scale-factor can not be computed.")
-            cd <- as.numeric(NA)
-        } else {
+  "lavaan WARNING: information matrix of the M01 model is not positive definite.\n")
+#"                  As a result, the scale-factor can not be computed.")
+            #cd <- as.numeric(NA)
+        } #else {
             # compute c_d
             # cd.01 <- (r0 * c01 - r1 * c0) / m ???
             cd <- (r0 * c0 - r1 * c01) / m
-        }
+        #}
 
     } else {
         # M1 with M0 parameters (as in Satorra & Bentler 2010)
@@ -246,13 +254,13 @@ lav_test_diff_SatorraBentler2010 <- function(m1, m0, H1 = FALSE) {
                          symmetric=TRUE, only.values=TRUE)$values
         if(any(eigvals < -1 * .Machine$double.eps^(3/4))) {
             warning(
-  "lavaan WARNING: information matrix of the M10 model is not positive definite.\n",
-"                  As a result, the scale-factor can not be computed.")
-            cd <- as.numeric(NA)
-        } else {
+  "lavaan WARNING: information matrix of the M10 model is not positive definite.\n")
+#"                  As a result, the scale-factor can not be computed.")
+            #cd <- as.numeric(NA)
+        } #else {
             # compute c_d
             cd <- (r0 * c0 - r1 * c10) / m
-        }
+        #}
     }
 
     # compute scaled difference test
@@ -355,11 +363,17 @@ lav_test_diff_A <- function(m1, m0, method = "delta", reference = "H1") {
         # take into account equality constraints m0
         if(m0@Model@eq.constraints) {
             Delta0 <- Delta0 %*% m0@Model@eq.constraints.K
+        } else if(.hasSlot(m0@Model, "ceq.simple.only") &&
+                  m0@Model@ceq.simple.only) {
+            Delta0 <- Delta0 %*% t(m0@Model@ceq.simple.K)
         }
 
         # take into account equality constraints m1
         if(m1@Model@eq.constraints) {
             Delta1 <- Delta1 %*% m1@Model@eq.constraints.K
+        } else if(.hasSlot(m1@Model, "ceq.simple.only") &&
+                  m1@Model@ceq.simple.only) {
+            Delta1 <- Delta1 %*% t(m1@Model@ceq.simple.K)
         }
 
         #H <- solve(t(Delta1) %*% Delta1) %*% t(Delta1) %*% Delta0
